#include "GNUChessControler.hpp"

#ifdef Q_OS_LINUX
// Too much Linux dependant
#include <memory.h>
#include <unistd.h>
#include <cstdlib>
#include <cstdio>

#include <iostream>
#include <exception>
#include <stdexcept>

#include "Utils/MoveParser.hpp"

GNUChessControler::GNUChessControler()
    :m_forkID(-1),m_isStarted(false)
{
}

GNUChessControler::~GNUChessControler()
{
    // This command stop GNUChess
    sendCommand("quit");
    close(m_readPipe[0]);
    close(m_readPipe[1]);
    close(m_writePipe[0]);
    close(m_writePipe[1]);
}

void GNUChessControler::sendCommand(const std::string& command)const
{
    std::string input(command);
    input += "\n";

    write(m_writePipe[1],input.data(),input.length());
}

void GNUChessControler::start()
{
    /* create the pipe */
    if (pipe(m_readPipe) == -1)
    {
        perror("pipe");
        throw std::runtime_error("GNUChessController : Read pipe creation failed");
    }

    if (pipe(m_writePipe) == -1)
    {
        perror("pipe");
        throw std::runtime_error("GNUChessController : Write pipe creation failed");
    }

    /* create the child */
    if ((m_forkID = fork()) < 0)
    {
        perror("fork");
        throw std::runtime_error("GNUChessController : Fork creation failed");
    }

    // The child just start GNUChess
    // The input and output from GNUChess are connected to the parent
    if (m_forkID == 0)
    {
        close(m_readPipe[0]);
        close(m_writePipe[1]);

        // We connect our pipe with in and out of gnuchess
        dup2(m_writePipe[0], 0);
        close(m_writePipe[0]);
        dup2(m_readPipe[1],1);
        close(m_readPipe[1]);

        execlp("gnuchess", "gnuchess","-x",NULL);

        printf("GNUChess ended"); /* if execlp returns, it's an error */
        exit(0); // Kill that children
        // No pipe close since they are connected to stdin / stdout (so, the system will do what it is needed)
    }
    else
    {
        close(m_writePipe[0]);
        close(m_readPipe[1]);
    }

    // Parent (of fork) does not have to do anything "yet"
    m_isStarted = true;
}

Move<ChessBoard::Coords> GNUChessControler::getMove()
{
    const char* catchPhrase = "My move is:";
    Move<ChessBoard::Coords> m;
    std::string moveString;
    char buffer[200] = {0};
    char* moveData = NULL;
    int moveSize = 0;

    do
    {
        memset(buffer,0,200);

        read(m_readPipe[0],buffer,199*sizeof(char));
        std::cout << "Buffer Out : '" << buffer << "'" << std::endl;

        moveData = strstr(buffer,catchPhrase);
        if ( moveData )
        {
            moveData += strlen(catchPhrase);
            while(*(moveData+moveSize) != '\n')
            {
                moveString.push_back(*(moveData+moveSize));
                moveSize++;
            }

            m = MoveParser::parse(moveString,m_colorControlled);
        }
    }while(!m.isLegal());

    return m;
}

void GNUChessControler::notify(const Move<ChessBoard::Coords> & move)
{
    if ( move.color() != m_colorControlled )
    {
        std::string command;
        command.push_back(move.start().column +'a');
        command.push_back(move.start().line +'1');
        command.push_back(move.dest().column +'a');
        command.push_back(move.dest().line +'1');
        if ( move.hasProperty("promotion") )
        {
            command.push_back(move.getProperty("promotion"));
        }

        std::cout << "Command send : " << command << std::endl;

        sendCommand(command);
    }
}

#endif
