#include "QtGNUChessControler.hpp"

#include <QDebug>
#include <QMetaType>
#include <QFile>
#include <QDir>

#include "Utils/MoveParser.hpp"

#if defined(Q_OS_LINUX)
    const QString QtGNUChessControler::m_gnuChessPath = "gnuchess";
#elif defined(Q_OS_WIN32)
    const QString QtGNUChessControler::m_gnuChessPath = "./data/bin/win/gnuchess.exe";
#else
    const QString QtGNUChessControler::m_gnuChessPath = "gnuchess";
#endif

QtGNUChessControler::QtGNUChessControler()
    :QObject(),m_isStarted(false)
{
    qRegisterMetaType<QProcess::ProcessError>("ProcessError");
}

QtGNUChessControler::~QtGNUChessControler()
{
    // This command stop GNUChess
    sendCommand("quit");
    m_process.waitForFinished(1000);
    // m_process.kill();
}

void QtGNUChessControler::sendCommand(const std::string& command)
{
    std::string input(command);
    input += "\n";

    qDebug() << "Send to GNUChess : " << input.c_str();
    m_process.write(input.c_str());
    m_process.waitForBytesWritten();
    qDebug() << "Sent";
}

void QtGNUChessControler::errorOccured(QProcess::ProcessError pe)
{
    qDebug() << "Error occured : " << pe;
    qDebug() << m_process.errorString();
}

void QtGNUChessControler::start()
{
    QStringList arguments;
    arguments << "-x";
    connect(&m_process,SIGNAL(error(QProcess::ProcessError)),this,SLOT(errorOccured(QProcess::ProcessError)));

    m_process.setProcessChannelMode(QProcess::MergedChannels);
    m_process.start(m_gnuChessPath,arguments,QIODevice::ReadWrite);
    m_process.waitForStarted();

    m_isStarted = true;
}

Move<ChessBoard::Coords> QtGNUChessControler::getMove()
{
    const char* catchPhrase = "My move is:";
    const int catchLength = strlen(catchPhrase);
    Move<ChessBoard::Coords> m;
    QString dataString;

    m_process.waitForBytesWritten(10000);

    do
    {
        qDebug() << "Strating to wait";
        m_process.waitForReadyRead(3000);
        qDebug() << "State : " << m_process.state();
        qDebug() << "ReadReady";
        QByteArray data = m_process.readAll();
        qDebug() << "Read (data) : " << data;
        dataString = QString(data);
        qDebug() << "Read : '" << dataString << "'";


        int index = dataString.indexOf(catchPhrase);
        if ( index != -1 )
        {
            int indexEnd = dataString.indexOf("\n",index);
            index += catchLength;

            m = MoveParser::parse(dataString.mid(index,indexEnd-index).toStdString(),m_colorControlled);
        }
    }while(!m.isLegal());

    return m;
}

void QtGNUChessControler::notify(const Move<ChessBoard::Coords> & move)
{
    if ( move.color() != m_colorControlled )
    {
        std::string command;
        command.push_back(move.start().column +'a');
        command.push_back(move.start().line +'1');
        command.push_back(move.dest().column +'a');
        command.push_back(move.dest().line +'1');
        if ( move.hasProperty("promotion") )
        {
            command.push_back(move.getProperty("promotion"));
        }

        sendCommand(command);
    }
}

bool QtGNUChessControler::checkAvailability()
{
    return QFile::exists(QString(QtGNUChessControler::m_gnuChessPath));
}
