#ifndef ABSTRACTGAME_HPP
#define ABSTRACTGAME_HPP

#include <string>

class AbstractGame
{
public:

    /*!
     */
    virtual ~AbstractGame() {}

    /*! Plays a game
     *  \return true if the game achieved in a normal state
     */
    virtual bool play()=0;

    /*! Gets if the game finished in a victory
     *  \return
     */
    virtual bool isVictory()const=0;

    /*! Gets the color of the winning player
     *  \return
     */
    virtual int getVictoriousColor()const=0;

    /*! Gets if the game finished in a draw
     *  \return
     */
    virtual bool isDraw()const=0;

    /*! Loads an history from a file named \a fileName
     *  \param fileName
     *  \return false if it was not possible to load the file
     */
    virtual bool loadHistory(const std::string& fileName)=0;

    /*! Saves the game history in a file \a fileName
     *  \param fileName
     *  \return false if it was not possible to save the file
     */
    virtual bool saveHistory(const std::string& fileName)=0;
};

/*! \class AbstractGame AbstractGame.hpp Game/AbstractGame.hpp
 *  \brief Base of all game that we can play
 *
 *  An abstract game is nothing else than an object that you can \c play() with.
 */

#endif
