#ifndef HISTORYLOADER_HPP
#define HISTORYLOADER_HPP

#include <string>

#include "Game/Move.hpp"
#include "Game/Boards/ChessBoard.hpp"
#include "History.hpp"

template <typename DataType>
class HistoryLoader
{
public:

    /*! Load chess moves from a file
     *  \param fileName the file name where to save the data
     *  \param error false if the file could not be read
     *  \return the history
     */
    static History<DataType> load(const std::string& fileName, bool& error);
};

template <>
class HistoryLoader<Move<ChessBoard::Coords> >
{
public:

    /*! Load chess moves from a file
     *  \param fileName the file name where to save the data
     *  \param error false if the file could not be read
     *  \return the history
     */
    static History<Move<ChessBoard::Coords> > load(const std::string& fileName, bool& error);
};

/*! \class HistoryLoader HistoryLoader.hpp Game/HistoryLoader.hpp
 *  \brief Generic HistoryLoader
 *
 *  loads an history from a file
 */

/*! \class HistoryLoader<Move<ChessBoard::Coords> > HistoryLoader.hpp Game/HistoryLoader.hpp
 *  \brief Specific history loader for chess game moves
 */

#endif
