#ifndef RULES_HPP
#define RULES_HPP

#include "Game/History.hpp"
#include "Game/Move.hpp"

template <typename BoardType>
class Rules
{
    typedef typename BoardType::Coords Coords;

private:

public:

    /*! Set the initial pieces on board
     *  \param cb the board to set
     */
    virtual void init(BoardType& cb)=0;

    /*! Check if the \a move is correct
     *  \param cb the board where the move should be checked
     *  \param move
     *  \return true if the move is legit
     */
    virtual bool check(const BoardType& cb, Move<Coords>& move)const=0;

    /*! Check if the game is in a victory condition
     *  \param cb the board of the game
     *  \param color the color for who to check the victory
     *  \return true if \a color wins
     */
    virtual bool checkVictory(const BoardType& cb, const int color)const=0;

    /*! Check if the game is in a draw condition
     *  \param history the game history to check if redondant move are
     *  \return true if the game is in draw
     */
    virtual bool checkDraw(const History<Move<Coords> >& history)const=0;

    /*! Apply a \a move
     *  \param cb the board of the game
     *  \param move
     *  \return
     */
    virtual void move(const BoardType& cb, const Move<Coords>& move)=0;
};

class RulesUtils
{
public:
    static const int PT_NONE = 0;       /*!< ID that defines the lack of piece */

    /*! Checks if the \a piece is valid and is a real piece (not PT_NONE)
     *  \param piece
     *  \return true if it is a valid piece
     */
    static bool isAPiece(const Piece& piece);
};

/*! \class Rules Rules.hpp Game/Rules/Rules.hpp
 *  \brief Base implementation for game rules
 *
 *  The rules are a set of laws that the player has to
 *  follow. This defines the initial position of the game,
 *  if a move a legal and when one of the player wins (or draw).
 *  These rules are described using the functions init(), check()
 *  checkVictory() and checkDraw(). The final function move() is
 *  here to keep a track on how is going the game.
 */

/*! \class RulesUtils RulesUtils.hpp Game/Rules/RulesUtils.hpp
 *  \brief Additional tools to apply rules
 */

#endif
