#include "GameFactory.hpp"

#include "Game/AbstractGame.hpp"
#include "Game/Game.hpp"
#include "Game/Rules/ChessRules.hpp"
#include "Controlers/QtControler.hpp"
#include "Controlers/QtGNUChessControler.hpp"

#include <iostream>
#include <cassert>

template <typename GameType>
void createControlers(GameType* pGame, const std::vector<int>& controlerTypes)
{
    for ( size_t i = 0 ; i < controlerTypes.size() ; i++ )
    {
        switch(controlerTypes[i])
        {
            case GameFactory::CT_GNUChess:
                pGame->addControler(new QtGNUChessControler());
                break;
            case GameFactory::CT_Player:
                pGame->addControler(new QtControler());
                break;
            default:
                break;
        }
    }
}

int GameFactory::getNbControlersNeeded(int gameType)
{
    switch (gameType)
    {
        case GT_Chess:
            return ChessRules::NB_PLAYERS;
            break;
        default:
            assert(false);
            break;
    }
}

AbstractGame* GameFactory::createGame(int gameType, const std::vector<int>& controlerTypes)
{
    if ( GameFactory::getNbControlersNeeded(gameType) != static_cast<int>(controlerTypes.size()) )
    {
        std::cout << "Not enough controlers for this game" << std::endl;
        assert(false);
        return NULL;
    }

    AbstractGame* pAbstractGame = NULL;
    switch (gameType)
    {
        case GT_Chess:
        {
            Game<ChessBoard, ChessRules>* pGame = new Game<ChessBoard, ChessRules>();
            createControlers<Game<ChessBoard, ChessRules> >(pGame,controlerTypes);
            pAbstractGame = pGame;
            break;
        }
        default:
            break;
    }

    return pAbstractGame;
}
