#ifndef GAMELAUNCHER_HPP
#define GAMELAUNCHER_HPP

#include <QThread>

class AbstractGame;

class GameLauncher : public QThread
{
    Q_OBJECT

private:
    AbstractGame* m_pGame; /*!< Game to be played in thread */

public:

    /*!
     */
    GameLauncher(AbstractGame* pGame)
        :m_pGame(pGame)
    {

    }

    /*! Threaded function
     */
    void run();

    /*! signal when the game finished with a draw */
    Q_SIGNAL void gameDraw();

    /*! signal when the game finished with a victory of \a color
     *  \param color the winning player
     */
    Q_SIGNAL void gameVictory(int color);
};

/*! \class GameLauncher GameLauncher.hpp GameLauncher.hpp
 *  \brief Thread for the game
 *
 *  The game is independant of the user interface (only
 *  attached together by the listener / notifier). Since we don't
 *  want to have an interface stucked by the game or opposite, the
 *  game is started using an indepent thread.
 *  \warning it seems that can be a bit tricky, since the notify() code
 *  is runned by this thread, even if the UI is done in another thread.
 *  To avoid this UI can launch signals to notify the UI thread of the changement
 */


#endif
