#ifndef QT2DPIECE_HPP
#define QT2DPIECE_HPP

#include <QGraphicsSvgItem>

#include <QGraphicsSceneDragDropEvent>

class Qt2DPiece : public QGraphicsSvgItem
{
    Q_OBJECT

private:

    int m_color; /*!< */

    /*! Variables about the drag and drop system */
    QPointF m_initialPos;
    QPointF m_initialPiecePos;
    QPointF m_offsetPos;

    bool m_rightButton; /*!< true when the right button of the mouse is used */

protected:

    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);

public:
    /*!
     *  \param fileName the name of the file to load
     *  \param parent
     */
    explicit Qt2DPiece(const QString& fileName,int color, QGraphicsItem* parent=NULL);

    /*! Resets the Piece position
     */
    ~Qt2DPiece();

    /*! Resets the Piece position
     */
    void reset();

    /*! Gets the color associated to the piece
     */
    int color()const { return m_color; }

    Q_SIGNAL void pieceMoved(QPointF start, QPointF end, int color);
    Q_SIGNAL void requestHelper(QPointF pos, int color);
    Q_SIGNAL void stopHelper();
};

/*! \class Qt2DPiece Qt2DPiece.hpp Renderers/Qt2DPiece.hpp
 *  \brief Element of Qt interface to represent a Piece
 *
 *  Displays a SVG picture and allow the user to move it using drag & drop.
 *  Additionnaly, the right click will send the signals requestHelper() and stopHelper() at release.
 *  The Piece also know about the color actually implementing, that you can get with color()
 *  Finally, if a user move is incorrect, the piece can go back to the original position using reset()
 *  \sa Qt2DDisplay
 */

#endif
