#ifndef RENDERER_HPP
#define RENDERER_HPP

#include "Game/Move.hpp"
#include "Utils/Listener.hpp"

#include <cassert>

template <typename BoardType>
class Renderer : public Listener<Move<typename BoardType::Coords> >
{
    typedef typename BoardType::Coords Coords;

protected:

    const BoardType *m_pBoard;      /*!< Board to display */

    /*! called when a board is set using setBoard
     */
    virtual void boardChanged() {}

public:

    /*! Defines the board to display
     *  Can be changed and the fly.
     *  \param pBoard the new board
     */
    void setBoard(const BoardType* pBoard)
    {
        assert(pBoard);

        m_pBoard = pBoard;
        boardChanged();
    }

    /*! do a draw call
     */
    virtual void display()
    {
        displayBoard();
    }

    /*! draw call to display the board
     */
    virtual void displayBoard()=0;

    /*! do a draw call for a piece
     *  \param position on the board of the piece to draw
     */
    virtual void displayPiece(const Coords& position)=0;
};

/*! \class Renderer Renderer.hpp Renderers/Renderer.hpp
 *  \brief Base class for renderers
 *
 *  A renderer must display the game (using the board).
 *  The board to view is set by using \c setBoard(). The function display() will print the game using subfunctions \c displayBoard()
 *  and \c displayPiece().
 *  The Renderer is a listener to watch all moves accepted (and so, update the view)
 */

#endif
