#ifndef GAMEWIDGET_HPP
#define GAMEWIDGET_HPP

#include <QMainWindow>

#include "Renderers/Qt2DDisplay.hpp"

class AbstractGame;
class GameLauncher;

class GameWidget : public QMainWindow
{
    Q_OBJECT

private:
    GameLauncher* m_pGameThread;    /*!< */
    AbstractGame* m_pGame;          /*!< */

    Qt2DDisplay* m_pRenderer;       /*!< */

    QAction* m_pSaveAction;         /*!< Action for the save menu (used to disable / enable it) */

    /*! \brief Deletes the class element to prepare it for a new game
     */
    void destroyGame();

    /*! \brief Creates the controlers, following \a controlerTypes
     *  The \a controlerTypes gives the ID to create the controlers that
     *  will be added to the game \a pGame.
     *  \param pGame
     *  \param controlerTypes
     */
    template <typename GameType>
    void createControlers(GameType* pGame, const std::vector<int>& controlerTypes);

    /*! \brief Creates a new game of type \a gameType with \a controlerTypes
     */
    void createGame(int gameType, const std::vector<int>& controlerTypes);

    /*! \brief the game finished with a draw
     */
    Q_SLOT void draw();

    /*! \brief the game finished with a victory for \a color
     *  \param color
     */
    Q_SLOT void victory(int color);

public:
    /*!
     *  \param parent
     */
    GameWidget(QWidget* parent=NULL);

    /*!
     */
    ~GameWidget()
    {
        destroyGame();
    }

    /*! \ßrief Gets the renderer
     *  \return
     */
    Qt2DDisplay* getRenderer() { return m_pRenderer; }

    /*! \brief Slot for new game button
     */
    Q_SLOT void newGame();

    /*! \brief Slot for load game button
     */
    Q_SLOT void loadGame();

    /*! \brief Slot for save game button
     */
    Q_SLOT void saveGame();
};

/*! \class GameWidget GameWidget.hpp UI/GameWidget.hpp
 *  \brief Main game interface
 *
 *  This widget integrates the game scene as main item in it.
 *  Moreover, this widget does the link between the game and the interface
 *  It does create the game after calling the NewGameDialog. It also gives
 *  some menus to give more control to the game.
 *  The game also implements button to load and save history.
 *  \todo The problem is that this class also implements a game factory
 *  \sa GameLauncher, AbstractGame, Qt2DDisplay
 */

#endif
