#include "NewGameDialog.hpp"
#include "ui_NewGameDialog.h"

#include "GameFactory.hpp"

#include "Controlers/QtGNUChessControler.hpp"

#include <QLabel>

#include <cassert>

NewGameDialog::NewGameDialog(QWidget *parent) :
    QDialog(parent),m_pUI(new Ui::NewGameDialog)
{
    m_pUI->setupUi(this);

    m_pUI->gameTypeCombo->addItem("Chess");
    reloadGameControls(0);
}

NewGameDialog::~NewGameDialog()
{
   delete m_pUI;
}

void NewGameDialog::fillComboControler(QComboBox* pCombo)
{
    if ( QtGNUChessControler::checkAvailability() )
    {
        pCombo->addItem("GNUChess");
    }
    pCombo->addItem("Player");
}

void NewGameDialog::reloadGameControls(int gameType)
{
    int controlersNeeded = GameFactory::getNbControlersNeeded(gameType);
    int actualLayoutCount = m_pUI->playersVLayout->count();

    if ( controlersNeeded == actualLayoutCount )
    {
        return;
    }

    // Need to add
    while ( actualLayoutCount < controlersNeeded )
    {
        QHBoxLayout* horizontalLayout = new QHBoxLayout();
        QLabel* label = new QLabel(QString("Player %1 :").arg(actualLayoutCount+1),m_pUI->verticalLayoutWidget);
        label->setObjectName(QString::fromUtf8("label_2"));

        horizontalLayout->addWidget(label);

        QComboBox* comboBox = new QComboBox(m_pUI->verticalLayoutWidget);
        comboBox->setObjectName(QString::fromUtf8("comboBox_2"));
        fillComboControler(comboBox);

        horizontalLayout->addWidget(comboBox);
        m_pUI->playersVLayout->addLayout(horizontalLayout);

        actualLayoutCount++;
    }
    while ( actualLayoutCount > controlersNeeded )
    {
        QLayoutItem* pItem = m_pUI->playersVLayout->itemAt(actualLayoutCount-1);
        m_pUI->playersVLayout->removeItem(pItem);
        delete pItem;

        actualLayoutCount--;
    }
}

int NewGameDialog::getGameType()const
{
    return m_pUI->gameTypeCombo->currentIndex();
}

std::vector<int> NewGameDialog::getControlerTypes()const
{
    std::vector<int> controlersTypes;
    int actualLayoutCount = m_pUI->playersVLayout->count();

    for ( int i = 0 ; i < actualLayoutCount ; i++ )
    {
        QLayoutItem* pItem = m_pUI->playersVLayout->itemAt(i);
        // Normally the item is a QHBoxLayout
        QHBoxLayout* pBox = dynamic_cast<QHBoxLayout*>(pItem->layout());
        if ( pBox )
        {
            QLayoutItem* pObjectItem = pBox->itemAt(1);
            QComboBox* pCombo = dynamic_cast<QComboBox*>(pObjectItem->widget());
            if ( pCombo )
            {
                controlersTypes.push_back(pCombo->currentIndex());
            }
        }
    }

    return controlersTypes;
}
